@@ -9,11 +9,11 @@ Google Spreadsheets client library.
 """
 
 
-__version__ = '3.6.0'
+__version__ = '4.0.0'
 __author__ = 'Anton Burnashev'
 
 
-from .auth import oauth, service_account
+from .auth import oauth, service_account, service_account_from_dict
 from .client import Client
 from .models import Spreadsheet, Worksheet, Cell
 
@@ -31,7 +31,7 @@ def authorize(credentials, client_class=Client):
     """Login to Google API using OAuth2 credentials.
     This is a shortcut function which
     instantiates `client_class`.
-    By default :class:`gspread.client.Client` is used.
+    By default :class:`gspread.Client` is used.
 
     :returns: `client_class` instance.
     """
