@@ -36,8 +36,11 @@ class Client(object):
     """
 
     def __init__(self, auth, session=None):
-        self.auth = convert_credentials(auth)
-        self.session = session or AuthorizedSession(self.auth)
+        if auth is not None:
+            self.auth = convert_credentials(auth)
+            self.session = session or AuthorizedSession(self.auth)
+        else:
+            self.session = session
 
     def login(self):
         from google.auth.transport.requests import Request
@@ -86,6 +89,7 @@ class Client(object):
             'pageSize': 1000,
             'supportsAllDrives': True,
             'includeItemsFromAllDrives': True,
+            'fields': 'kind,nextPageToken,files(id,name,createdTime,modifiedTime)',
         }
 
         while page_token is not None:
@@ -110,8 +114,7 @@ class Client(object):
         :raises gspread.SpreadsheetNotFound: if no spreadsheet with
                                              specified `title` is found.
 
-        >>> c = gspread.authorize(credentials)
-        >>> c.open('My fancy spreadsheet')
+        >>> gc.open('My fancy spreadsheet')
         """
         try:
             properties = finditem(
@@ -132,8 +135,7 @@ class Client(object):
         :param str key: A key of a spreadsheet as it appears in a URL in a browser.
         :returns: a :class:`~gspread.models.Spreadsheet` instance.
 
-        >>> c = gspread.authorize(credentials)
-        >>> c.open_by_key('0BmgG6nO_6dprdS1MN3d3MkdPa142WFRrdnRRUWl1UFE')
+        >>> gc.open_by_key('0BmgG6nO_6dprdS1MN3d3MkdPa142WFRrdnRRUWl1UFE')
         """
         return Spreadsheet(self, {'id': key})
 
@@ -147,8 +149,7 @@ class Client(object):
         :raises gspread.SpreadsheetNotFound: if no spreadsheet with
                                              specified `url` is found.
 
-        >>> c = gspread.authorize(credentials)
-        >>> c.open_by_url('https://docs.google.com/spreadsheet/ccc?key=0Bm...FE&hl')
+        >>> gc.open_by_url('https://docs.google.com/spreadsheet/ccc?key=0Bm...FE&hl')
         """
         return self.open_by_key(extract_id_from_url(url))
 
@@ -190,14 +191,18 @@ class Client(object):
             'mimeType': 'application/vnd.google-apps.spreadsheet',
         }
 
+        params = {
+            "supportsAllDrives": True,
+        }
+
         if folder_id is not None:
             payload['parents'] = [folder_id]
 
-        r = self.request('post', DRIVE_FILES_API_V3_URL, json=payload)
+        r = self.request('post', DRIVE_FILES_API_V3_URL, json=payload, params=params)
         spreadsheet_id = r.json()['id']
         return self.open_by_key(spreadsheet_id)
 
-    def copy(self, file_id, title=None, copy_permissions=False):
+    def copy(self, file_id, title=None, copy_permissions=False, folder_id=None):
         """Copies a spreadsheet.
 
         :param str file_id: A key of a spreadsheet to copy.
@@ -206,14 +211,18 @@ class Client(object):
         :param bool copy_permissions: (optional) If True, copy permissions from
             the original spreadsheet to the new spreadsheet.
 
+        :param str folder_id: Id of the folder where we want to save
+            the spreadsheet.
+
         :returns: a :class:`~gspread.models.Spreadsheet` instance.
 
         .. versionadded:: 3.1.0
 
         .. note::
 
-           In order to use this method, you need to add
-           ``https://www.googleapis.com/auth/drive`` to your oAuth scope.
+           If you're using custom credentials without the Drive scope, you need to add 
+           ``https://www.googleapis.com/auth/drive`` to your OAuth scope in order to use 
+           this method.
 
            Example::
 
@@ -222,7 +231,7 @@ class Client(object):
                   'https://www.googleapis.com/auth/drive'
               ]
 
-           Otherwise you will get an ``Insufficient Permission`` error
+           Otherwise, you will get an ``Insufficient Permission`` error
            when you try to copy a spreadsheet.
 
         """
@@ -232,13 +241,17 @@ class Client(object):
             'title': title,
             'mimeType': 'application/vnd.google-apps.spreadsheet',
         }
+
+        if folder_id is not None:
+            payload['parents'] = [{'id': folder_id}]
+        
         params = {'supportsAllDrives': True}
         r = self.request('post', url, json=payload, params=params)
         spreadsheet_id = r.json()['id']
 
         new_spreadsheet = self.open_by_key(spreadsheet_id)
 
-        if copy_permissions:
+        if copy_permissions is True:
             original = self.open_by_key(file_id)
 
             permissions = original.list_permissions()
