@@ -14,6 +14,7 @@ from .utils import (
     a1_to_rowcol,
     rowcol_to_a1,
     cast_to_a1_notation,
+    numericise,
     numericise_all,
     finditem,
     fill_gaps,
@@ -30,6 +31,7 @@ from .urls import (
     SPREADSHEET_URL,
     SPREADSHEET_VALUES_URL,
     SPREADSHEET_VALUES_BATCH_URL,
+    SPREADSHEET_VALUES_BATCH_CLEAR_URL,
     SPREADSHEET_BATCH_UPDATE_URL,
     SPREADSHEET_VALUES_APPEND_URL,
     SPREADSHEET_VALUES_CLEAR_URL,
@@ -38,17 +40,14 @@ from .urls import (
     SPREADSHEET_VALUES_BATCH_UPDATE_URL,
     SPREADSHEET_SHEETS_COPY_TO_URL,
 )
-
-try:
-    unicode
-except NameError:
-    basestring = unicode = str
+from gspread import utils
 
 
 class ValueRange(list):
     @classmethod
     def from_json(cls, json):
-        new_obj = cls(json['values'])
+        values = json.get('values', [])
+        new_obj = cls(values)
         new_obj._json = {
             'range': json['range'],
             'majorDimension': json['majorDimension'],
@@ -102,6 +101,32 @@ class Spreadsheet(object):
         """Spreadsheet URL."""
         return SPREADSHEET_DRIVE_URL % self.id
 
+    @property
+    def creationTime(self):
+        """Spreadsheet Creation time."""
+        try:
+            return self._properties['createdTime']
+        except KeyError:
+            # Filter the list using the name to reduce the request size
+            # Filter the item using the unique ID to ensure we update the exacte same item
+            metadata = utils.finditem(lambda x: x["id"] == self.id,
+                                    self.client.list_spreadsheet_files(self.title))
+            self._properties.update(metadata)
+            return self._properties['createdTime']
+
+    @property
+    def lastUpdateTime(self):
+        """Spreadsheet Creation time."""
+        try:
+            return self._properties['modifiedTime']
+        except KeyError:
+            # Filter the list using the name to reduce the request size
+            # Filter the item using the unique ID to ensure we update the exacte same item
+            metadata = utils.finditem(lambda x: x["id"] == self.id,
+                                    self.client.list_spreadsheet_files(self.title))
+            self._properties.update(metadata)
+            return self._properties['modifiedTime']
+
     @property
     def updated(self):
         """.. deprecated:: 2.0
@@ -177,6 +202,11 @@ class Spreadsheet(object):
         r = self.client.request('post', url)
         return r.json()
 
+    def values_batch_clear(self, params=None, body=None):
+        url = SPREADSHEET_VALUES_BATCH_CLEAR_URL % self.id
+        r = self.client.request('post', url, params=params, json=body)
+        return r.json()
+
     def values_get(self, range, params=None):
         """Lower-level method that directly calls `spreadsheets.values.get <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/get>`_.
 
@@ -241,7 +271,7 @@ class Spreadsheet(object):
         return r.json()
 
     def _spreadsheets_get(self, params=None):
-        """A method stub that directly calls `spreadsheets.batchUpdate <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/get>`_. """
+        """A method stub that directly calls `spreadsheets.get <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/get>`_. """
         url = SPREADSHEET_URL % self.id
         r = self.client.request('get', url, params=params)
         return r.json()
@@ -272,13 +302,15 @@ class Spreadsheet(object):
         :param index: An index of a worksheet. Indexes start from zero.
         :type index: int
 
-        :returns: an instance of :class:`gspread.models.Worksheet`
-                  or `None` if the worksheet is not found.
+        :returns: an instance of :class:`gspread.models.Worksheet`.
 
-        Example. To get first worksheet of a spreadsheet:
+        :raises:
+            WorksheetNotFound: if can't find the worksheet
+
+        Example. To get third worksheet of a spreadsheet:
 
         >>> sht = client.open('My fancy spreadsheet')
-        >>> worksheet = sht.get_worksheet(0):rtype: dict
+        >>> worksheet = sht.get_worksheet(2)
         """
         sheet_data = self.fetch_sheet_metadata()
 
@@ -286,7 +318,33 @@ class Spreadsheet(object):
             properties = sheet_data['sheets'][index]['properties']
             return Worksheet(self, properties)
         except (KeyError, IndexError):
-            return None
+            raise WorksheetNotFound("index {} not found".format(index))
+    
+    def get_worksheet_by_id(self, id):
+        """Returns a worksheet with specified `worksheet id`.
+
+        :param id: The id of a worksheet. it can be seen in the url as the value of the parameter 'gid'.
+        :type id: int
+
+        :returns: an instance of :class:`gspread.models.Worksheet`.
+        :raises:
+            WorksheetNotFound: if can't find the worksheet
+
+        Example. To get the worksheet 123456 of a spreadsheet:
+
+        >>> sht = client.open('My fancy spreadsheet')
+        >>> worksheet = sht.get_worksheet_by_id(123456)
+        """
+        sheet_data = self.fetch_sheet_metadata()
+
+        try:
+            item = finditem(
+                lambda x: x['properties']['sheetId'] == id,
+                sheet_data['sheets'],
+            )
+            return Worksheet(self, item['properties'])
+        except (StopIteration, KeyError):
+            raise WorksheetNotFound("id {} not found".format(id))
 
     def worksheets(self):
         """Returns a list of all :class:`worksheets <gspread.models.Worksheet>`
@@ -305,6 +363,9 @@ class Spreadsheet(object):
 
         :returns: an instance of :class:`gspread.models.Worksheet`.
 
+        :raises:
+            WorksheetNotFound: if can't find the worksheet
+
         Example. Getting worksheet named 'Annual bonuses'
 
         >>> sht = client.open('Sample one')
@@ -661,7 +722,8 @@ class Worksheet(object):
     def range(self, name):
         """Returns a list of :class:`Cell` objects from a specified range.
 
-        :param name: A string with range value in A1 notation, e.g. 'A1:A5'.
+        :param name: A string with range value in A1 notation (e.g. 'A1:A5')
+                     or the named range to fetch.
         :type name: str
 
         Alternatively, you may specify numeric boundaries. All values
@@ -681,12 +743,29 @@ class Worksheet(object):
             >>> # Same with numeric boundaries
             >>> worksheet.range(1, 1, 7, 2)
             [<Cell R1C1 "42">, ...]
+
+            >>> # Named ranges work as well
+            >>> worksheet.range('NamedRange')
+            [<Cell R1C1 "42">, ...]
         """
         range_label = absolute_range_name(self.title, name)
 
         data = self.spreadsheet.values_get(range_label)
 
-        start, end = name.split(':')
+        start = None
+        end = None
+
+        if ':' not in name:
+            range_name = data.get('range', None)
+            if range_name and '!' in range_name:
+                coordinates = range_name.split('!')[1]
+                if ':' in coordinates:
+                    start, end = coordinates.split(':')
+            if not start or not end:
+                raise ValueError('Cannot retrieve range coordinates')
+        else:
+            start, end = name.split(':')
+
         (row_offset, column_offset) = a1_to_rowcol(start)
         (last_row, last_column) = a1_to_rowcol(end)
 
@@ -704,31 +783,106 @@ class Worksheet(object):
             for j, value in enumerate(row)
         ]
 
-    def get_all_values(self, value_render_option='FORMATTED_VALUE'):
-        """Returns a list of lists containing all cells' values as strings.
+    @accepted_kwargs(
+        major_dimension=None,
+        value_render_option=None,
+        date_time_render_option=None,
+    )
+    def get_values(self, range_name=None, **kwargs):
+        """Returns a list of lists containing all values from specified range.
 
-        :param value_render_option: (optional) Determines how values should be
-                                    rendered in the the output. See
-                                    `ValueRenderOption`_ in the Sheets API.
-        :type value_render_option: str
+        By default values are returned as strings. See ``value_render_option``
+        to change the default format.
+
+        :param str range_name: (optional) Cell range in the A1 notation or
+            a named range. If not specified the method returns values from all
+            non empty cells.
+
+        :param str major_dimension: (optional) The major dimension of the
+            values. Either ``ROWS`` or ``COLUMNS``. Defaults to ``ROWS``
+
+        :param str value_render_option: (optional) Determines how values should
+            be rendered in the the output. See `ValueRenderOption`_ in
+            the Sheets API.
+
+            Possible values are:
+
+            ``FORMATTED_VALUE``
+                (default) Values will be calculated and formatted according
+                to the cell's formatting. Formatting is based on the
+                spreadsheet's locale, not the requesting user's locale.
+
+            ``UNFORMATTED_VALUE``
+                Values will be calculated, but not formatted in the reply.
+                For example, if A1 is 1.23 and A2 is =A1 and formatted as
+                currency, then A2 would return the number 1.23.
+
+            ``FORMULA``
+                Values will not be calculated. The reply will include
+                the formulas. For example, if A1 is 1.23 and A2 is =A1 and
+                formatted as currency, then A2 would return "=A1".
 
         .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
 
+        :param str date_time_render_option: (optional) How dates, times, and
+            durations should be represented in the output. This is ignored if
+            ``value_render_option`` is ``FORMATTED_VALUE``. The default
+            ``date_time_render_option`` is ``SERIAL_NUMBER``.
+
         .. note::
 
             Empty trailing rows and columns will not be included.
+
+        Examples::
+
+            # Return all values from the sheet
+            worksheet.get_values()
+
+            # Return all values from columns "A" and "B"
+            worksheet.get_values('A:B')
+
+            # Return values from range "A2:C10"
+            worksheet.get_values('A2:C10')
+
+            # Return values from named range "my_range"
+            worksheet.get_values('my_range')
+
+            # Return unformatted values (e.g. numbers as numbers)
+            worksheet.get_values('A2:B4', value_render_option='UNFORMATTED_VALUE')
+
+            # Return cell values without calculating formulas
+            worksheet.get_values('A2:B4', value_render_option='FORMULA')
         """
-        range_name = absolute_range_name(self.title)
-
-        data = self.spreadsheet.values_get(
-            range_name, params={'valueRenderOption': value_render_option}
-        )
-
         try:
-            return fill_gaps(data['values'])
+            return fill_gaps(self.get(range_name, **kwargs))
         except KeyError:
             return []
 
+    @accepted_kwargs(
+        major_dimension=None,
+        value_render_option=None,
+        date_time_render_option=None,
+    )
+    def get_all_values(self, **kwargs):
+        """Returns a list of lists containing all cells' values as strings.
+
+        This is an alias to :meth:`~gspread.models.Worksheet.get_values`
+
+        .. note::
+
+            This is a legacy method.
+            Use :meth:`~gspread.models.Worksheet.get_values` instead.
+
+        Examples::
+
+            # Return all values from the sheet
+            worksheet.get_all_values()
+
+            # Is equivalent to
+            worksheet.get_values()
+        """
+        return self.get_values(**kwargs)
+
     def get_all_records(
         self,
         empty2zero=False,
@@ -736,6 +890,7 @@ class Worksheet(object):
         default_blank="",
         allow_underscores_in_numeric_literals=False,
         numericise_ignore=None,
+        value_render_option=None,
     ):
         """Returns a list of dictionaries, all of them having the contents of
         the spreadsheet with the head row as keys and each of these
@@ -743,7 +898,7 @@ class Worksheet(object):
         values.
 
         Cell values are numericised (strings that can be read as ints or floats
-        are converted), unless specificed in numericise_ignore
+        are converted), unless specified in numericise_ignore
 
         :param bool empty2zero: (optional) Determines whether empty cells are
             converted to zeros.
@@ -756,10 +911,15 @@ class Worksheet(object):
         :param list numericise_ignore: (optional) List of ints of indices of
             the row (starting at 1) to ignore numericising, special use
             of ['all'] to ignore numericising on all columns.
+        :param str value_render_option: (optional) Determines how values should
+            be rendered in the the output. See `ValueRenderOption`_ in
+            the Sheets API.
+
+        .. _ValueRenderOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueRenderOption
         """
         idx = head - 1
 
-        data = self.get_all_values()
+        data = self.get_all_values(value_render_option=value_render_option)
 
         # Return an empty list if the sheet doesn't have enough rows
         if len(data) <= idx:
@@ -874,8 +1034,21 @@ class Worksheet(object):
         """Updates many cells at once.
 
         :param list cell_list: List of :class:`Cell` objects to update.
-        :param str value_input_option: (optional) Determines how input data
-            should be interpreted. See `ValueInputOption`_ in the Sheets API.
+        :param str value_input_option: (optional) How the input data should be
+            interpreted. Possible values are:
+
+            ``RAW``
+                The values the user has entered will not be parsed and will be
+                stored as-is.
+
+            ``USER_ENTERED``
+                The values will be parsed as if the user typed them into the
+                UI. Numbers will stay as numbers, but strings may be converted
+                to numbers, dates, etc. following the same rules that are
+                applied when entering text into a cell via
+                the Google Sheets UI.
+
+            See `ValueInputOption`_ in the Sheets API.
 
         .. _ValueInputOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueInputOption
 
@@ -923,16 +1096,16 @@ class Worksheet(object):
             a named range.
 
         :param str major_dimension: (optional) The major dimension that results
-            should use.
+            should use. Either ``ROWS`` or ``COLUMNS``.
 
         :param str value_render_option: (optional) How values should be
             represented in the output. The default render option is
-            `FORMATTED_VALUE`.
+            ``FORMATTED_VALUE``.
 
         :param str date_time_render_option: (optional) How dates, times, and
             durations should be represented in the output. This is ignored if
-            value_render_option is FORMATTED_VALUE. The default dateTime render
-            option is `SERIAL_NUMBER`.
+            ``value_render_option`` is ``FORMATTED_VALUE``. The default
+            ``date_time_render_option`` is ``SERIAL_NUMBER``.
 
         Examples::
 
@@ -976,16 +1149,16 @@ class Worksheet(object):
             ranges.
 
         :param str major_dimension: (optional) The major dimension that results
-            should use.
+            should use. Either ``ROWS`` or ``COLUMNS``.
 
         :param str value_render_option: (optional) How values should be
             represented in the output. The default render option
-            is `FORMATTED_VALUE`.
+            is ``FORMATTED_VALUE``.
 
         :param str date_time_render_option: (optional) How dates, times, and
             durations should be represented in the output. This is ignored if
-            value_render_option is FORMATTED_VALUE. The default dateTime render
-            option is `SERIAL_NUMBER`.
+            value_render_option is ``FORMATTED_VALUE``. The default dateTime
+            render option is ``SERIAL_NUMBER``.
 
         .. versionadded:: 3.3
 
@@ -1021,24 +1194,31 @@ class Worksheet(object):
     def update(self, range_name, values=None, **kwargs):
         """Sets values in a cell range of the sheet.
 
-        :param str range_name: (optional) The A1 notation of the values
+        :param str range_name: The A1 notation of the values
             to update.
         :param list values: The data to be written.
-        :param str major_dimension: (optional) The major dimension that results
-            should use.
+
+        :param bool raw: The values will not be parsed by Sheets API and will
+            be stored as-is. For example, formulas will be rendered as plain
+            strings. Defaults to ``True``. This is a shortcut for
+            the ``value_input_option`` parameter.
+
+        :param str major_dimension: (optional) The major dimension of the
+            values. Either ``ROWS`` or ``COLUMNS``.
 
         :param str value_input_option: (optional) How the input data should be
-            interpreted.
+            interpreted. Possible values are:
 
-            Possible values are:
+            ``RAW``
+                The values the user has entered will not be parsed and will be
+                stored as-is.
 
-            RAW             The values the user has entered will not be parsed
-                            and will be stored as-is.
-            USER_ENTERED    The values will be parsed as if the user typed them
-                            into the UI. Numbers will stay as numbers, but
-                            strings may be converted to numbers, dates, etc.
-                            following the same rules that are applied when
-                            entering text into a cell via the Google Sheets UI.
+            ``USER_ENTERED``
+                The values will be parsed as if the user typed them into the
+                UI. Numbers will stay as numbers, but strings may be converted
+                to numbers, dates, etc. following the same rules that are
+                applied when entering text into a cell via
+                the Google Sheets UI.
 
         Examples::
 
@@ -1118,21 +1298,22 @@ class Worksheet(object):
             is a target range to update in A1 notation or a named range,
             and `values` is a list of lists containing new values.
 
-        :param str major_dimension: (optional) The major dimension that results
-            should use.
+        :param str major_dimension: (optional) The major dimension of the
+            values. Either ``ROWS`` or ``COLUMNS``.
 
         :param str value_input_option: (optional) How the input data should be
-            interpreted.
+            interpreted. Possible values are:
 
-            Possible values are:
+            ``RAW``
+                The values the user has entered will not be parsed and will be
+                stored as-is.
 
-            RAW             The values the user has entered will not be parsed
-                            and will be stored as-is.
-            USER_ENTERED    The values will be parsed as if the user typed them
-                            into the UI. Numbers will stay as numbers, but
-                            strings may be converted to numbers, dates, etc.
-                            following the same rules that are applied when
-                            entering text into a cell via the Google Sheets UI.
+            ``USER_ENTERED``
+                The values will be parsed as if the user typed them into the
+                UI. Numbers will stay as numbers, but strings may be converted
+                to numbers, dates, etc. following the same rules that are
+                applied when entering text into a cell via
+                the Google Sheets UI.
 
         Examples::
 
@@ -1400,7 +1581,7 @@ class Worksheet(object):
         self.resize(rows=self.row_count + rows)
 
     def add_cols(self, cols):
-        """Adds colums to worksheet.
+        """Adds columns to worksheet.
 
         :param cols: Number of new columns to add.
         :type cols: int
@@ -1428,7 +1609,7 @@ class Worksheet(object):
             reference.
         :param str table_range: (optional) The A1 notation of a range to search
             for a logical table of data. Values are appended after the last row
-            of the table. Examples: `A1` or `B2:D4`
+            of the table. Examples: ``A1`` or ``B2:D4``
 
         .. _ValueInputOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueInputOption
         .. _InsertDataOption: https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append#InsertDataOption
@@ -1455,13 +1636,14 @@ class Worksheet(object):
         :param list values: List of rows each row is List of values for
             the new row.
         :param str value_input_option: (optional) Determines how input data
-            should be interpreted. See `ValueInputOption`_ in the Sheets API.
+            should be interpreted. Possible values are ``RAW`` or
+            ``USER_ENTERED``. See `ValueInputOption`_ in the Sheets API.
         :param str insert_data_option: (optional) Determines how the input data
             should be inserted. See `InsertDataOption`_ in the Sheets API
             reference.
         :param str table_range: (optional) The A1 notation of a range to search
             for a logical table of data. Values are appended after the last row
-            of the table. Examples: `A1` or `B2:D4`
+            of the table. Examples: ``A1`` or ``B2:D4``
 
         .. _ValueInputOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueInputOption
         .. _InsertDataOption: https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append#InsertDataOption
@@ -1486,37 +1668,12 @@ class Worksheet(object):
         :param list values: List of values for the new row.
         :param int index: (optional) Offset for the newly inserted row.
         :param str value_input_option: (optional) Determines how input data
-            should be interpreted. See `ValueInputOption`_ in the Sheets API.
+            should be interpreted. Possible values are ``RAW`` or
+            ``USER_ENTERED``. See `ValueInputOption`_ in the Sheets API.
 
         .. _ValueInputOption: https://developers.google.com/sheets/api/reference/rest/v4/ValueInputOption
         """
-
-        body = {
-            "requests": [
-                {
-                    "insertDimension": {
-                        "range": {
-                            "sheetId": self.id,
-                            "dimension": "ROWS",
-                            "startIndex": index - 1,
-                            "endIndex": index,
-                        }
-                    }
-                }
-            ]
-        }
-
-        self.spreadsheet.batch_update(body)
-
-        range_label = absolute_range_name(self.title, 'A%s' % index)
-
-        data = self.spreadsheet.values_update(
-            range_label,
-            params={'valueInputOption': value_input_option},
-            body={'values': [values]},
-        )
-
-        return data
+        return self.insert_rows([values], index, value_input_option=value_input_option)
 
     def insert_rows(self, values, row=1, value_input_option='RAW'):
         """Adds multiple rows to the worksheet at the specified index and
@@ -1526,6 +1683,9 @@ class Worksheet(object):
             each containing one row's values. Widens the worksheet if there are
             more values than columns.
         :param int row: Start row to update (one-based). Defaults to 1 (one).
+        :param str value_input_option: (optional) Determines how input data
+            should be interpreted. Possible values are ``RAW`` or
+            ``USER_ENTERED``. See `ValueInputOption`_ in the Sheets API.
         """
         body = {
             "requests": [
@@ -1552,6 +1712,43 @@ class Worksheet(object):
 
         return self.spreadsheet.values_append(range_label, params, body)
 
+    def insert_cols(self, values, col=1, value_input_option='RAW'):
+        """Adds multiple new cols to the worksheet at specified index and
+        populates them with values.
+
+        :param list values: List of col lists. a list of lists, with the lists
+            each containing one col's values. Increases the number of rows
+            if there are more values than columns.
+        :param int col: Start col to update (one-based). Defaults to 1 (one).
+        :param str value_input_option: (optional) Determines how input data
+            should be interpreted. Possible values are ``RAW`` or
+            ``USER_ENTERED``. See `ValueInputOption`_ in the Sheets API.
+        """
+        body = {
+            "requests": [
+                {
+                    "insertDimension": {
+                        "range": {
+                            "sheetId": self.id,
+                            "dimension": "COLUMNS",
+                            "startIndex": col - 1,
+                            "endIndex": len(values) + col - 1,
+                        }
+                    }
+                }
+            ]
+        }
+
+        self.spreadsheet.batch_update(body)
+
+        range_label = absolute_range_name(self.title, rowcol_to_a1(1, col))
+
+        params = {'valueInputOption': value_input_option}
+
+        body = {'majorDimension': 'COLUMNS', 'values': values}
+
+        return self.spreadsheet.values_append(range_label, params, body)
+
     def delete_row(self, index):
         """Deletes the row from the worksheet at the specified index.
 
@@ -1591,6 +1788,8 @@ class Worksheet(object):
         :param int last_row: Last row number
         :param int last_col: Last column number
 
+        For both A1 and numeric notation:
+
         :param list editor_users_emails: (optional) The email addresses of
             users with edit access to the protected range.
         :param list editor_groups_emails: (optional) The email addresses of
@@ -1667,7 +1866,7 @@ class Worksheet(object):
         return self.spreadsheet.batch_update(body)
 
     def delete_rows(self, start_index, end_index=None):
-        """Deletes multi rows from the worksheet at the specified index.
+        """Deletes multiple rows from the worksheet at the specified index.
 
         :param int start_index: Index of a first row for deletion.
         :param int end_index: Index of a last row for deletion.
@@ -1686,7 +1885,7 @@ class Worksheet(object):
         return self.delete_dimension('ROWS', start_index, end_index)
 
     def delete_columns(self, start_index, end_index=None):
-        """Deletes multi columns from the worksheet at the specified index.
+        """Deletes multiple columns from the worksheet at the specified index.
 
         :param int start_index: Index of a first column for deletion.
         :param int end_index: Index of a last column for deletion.
@@ -1699,6 +1898,30 @@ class Worksheet(object):
         """Clears all cells in the worksheet."""
         return self.spreadsheet.values_clear(absolute_range_name(self.title))
 
+    def batch_clear(self, ranges):
+        """Clears multiple ranges of cells with 1 API call.
+
+        https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/batchClear
+
+        Examples::
+
+            worksheet.batch_clear(['A1:B1','my_range'])
+
+            # Note: named ranges are defined in the scope of
+            # a spreadsheet, so even if `my_range` does not belong to
+            # this sheet it is still updated
+
+        .. versionadded:: 3.8.0
+
+        """
+        ranges = [absolute_range_name(self.title, rng) for rng in ranges]
+
+        body = {'ranges': ranges}
+
+        response = self.spreadsheet.values_batch_clear(body=body)
+
+        return response
+
     def _finder(self, func, query, in_row=None, in_column=None):
         data = self.spreadsheet.values_get(absolute_range_name(self.title))
 
@@ -1709,7 +1932,7 @@ class Worksheet(object):
 
         cells = self._list_cells(values, in_row, in_column)
 
-        if isinstance(query, basestring):
+        if isinstance(query, str):
             match = lambda x: x.value == query
         else:
             match = lambda x: query.search(x.value)
@@ -1750,11 +1973,12 @@ class Worksheet(object):
         :param int in_row: (optional) One-based row number to scope the search.
         :param int in_column: (optional) One-based column number to scope
             the search.
+        :returns: the first matching cell or None otherwise
         """
         try:
             return self._finder(finditem, query, in_row, in_column)
         except StopIteration:
-            raise CellNotFound(query)
+            return None
 
     def findall(self, query, in_row=None, in_column=None):
         """Finds all cells matching the query.
@@ -1810,7 +2034,7 @@ class Worksheet(object):
         are passed, the filter will be limited to the given range.
 
         :param str name: A string with range value in A1 notation,
-            e.g. 'A1:A5'.
+            e.g. ``A1:A5``.
 
         Alternatively, you may specify numeric boundaries. All values
         index from 1 (one):
@@ -1896,13 +2120,13 @@ class Worksheet(object):
 
     @cast_to_a1_notation
     def merge_cells(self, name, merge_type="MERGE_ALL"):
-        """Merge cells. There are 3 merge types: MERGE_ALL, MERGE_COLUMNS,
-        and MERGE_ROWS.
+        """Merge cells. There are 3 merge types: ``MERGE_ALL``, ``MERGE_COLUMNS``,
+        and ``MERGE_ROWS``.
 
         :param str name: Range name in A1 notation, e.g. 'A1:A5'.
-        :param str merge_type: (optional) one of MERGE_ALL, MERGE_COLUMNS,
-            or MERGE_ROWS. Defaults to MERGE_ROWS. See `MergeType`_ in
-            the Sheets API reference.
+        :param str merge_type: (optional) one of ``MERGE_ALL``,
+            ``MERGE_COLUMNS``, or ``MERGE_ROWS``. Defaults to ``MERGE_ROWS``.
+            See `MergeType`_ in the Sheets API reference.
 
         Alternatively, you may specify numeric boundaries. All values
         index from 1 (one):
@@ -1928,6 +2152,112 @@ class Worksheet(object):
 
         return self.spreadsheet.batch_update(body)
 
+    def get_note(self, cell):
+        """Get the content of the note located at `cell`, or the empty string if the
+        cell does not have a note.
+
+        :param str cell: A string with cell coordinates in A1 notation,
+            e.g. 'D7'.
+        """
+        absolute_cell = absolute_range_name(self.title, cell)
+        url = SPREADSHEET_URL % (self.spreadsheet.id)
+        params = {
+            'ranges': absolute_cell,
+            'fields': 'sheets/data/rowData/values/note'
+        }
+        response = self.client.request('get', url, params=params)
+        response.raise_for_status()
+        response_json = response.json()
+
+        try:
+            note = response_json['sheets'][0]['data'][0]['rowData'][0]['values'][0]['note']
+        except (IndexError, KeyError):
+            note = ''
+
+        return note
+
+    def update_note(self, cell, content):
+        """Update the content of the note located at `cell`.
+
+        :param str cell: A string with cell coordinates in A1 notation,
+            e.g. 'D7'.
+        :param str note: The text note to insert.
+
+        .. versionadded:: 3.7
+        """
+
+        if not isinstance(content, str):
+            raise TypeError("Only string allowed as content for a note.")
+
+        (startRow, startColumn) = a1_to_rowcol(cell)
+
+        body = {
+            "requests": [
+                {
+                    "updateCells": {
+                        "range": {
+                            "sheetId": self.id,
+                            "startRowIndex": startRow - 1,
+                            "endRowIndex": startRow,
+                            "startColumnIndex": startColumn - 1,
+                            "endColumnIndex": startColumn,
+                        },
+                        "rows": [
+                            {
+                                "values": [
+                                    {
+                                        "note": content
+                                    }
+                                ]
+                            }
+                        ],
+                        "fields": "note"
+                    }
+                }
+            ]
+        }
+        self.spreadsheet.batch_update(body)
+
+    @cast_to_a1_notation
+    def insert_note(self, cell, content):
+        """Insert a note. The note is attached to a certain cell.
+
+        :param str cell: A string with cell coordinates in A1 notation,
+            e.g. 'D7'.
+        :param str content: The text note to insert.
+
+        Alternatively, you may specify numeric boundaries. All values
+        index from 1 (one):
+
+        :param int first_row: First row number
+        :param int first_col: First column number
+        :param int last_row: Last row number
+        :param int last_col: Last column number
+
+        .. versionadded:: 3.7
+        """
+        self.update_note(cell, content)
+
+    @cast_to_a1_notation
+    def clear_note(self, cell):
+        """Clear a note. The note is attached to a certain cell.
+
+        :param str cell: A string with cell coordinates in A1 notation,
+            e.g. 'D7'.
+
+        Alternatively, you may specify numeric boundaries. All values
+        index from 1 (one):
+
+        :param int first_row: First row number
+        :param int first_col: First column number
+        :param int last_row: Last row number
+        :param int last_col: Last column number
+
+        .. versionadded:: 3.7
+        """
+        # set the note to <empty string> will clear it
+        self.update_note(cell, "")
+
 
 class Cell(object):
     """An instance of this class represents a single cell
@@ -1965,9 +2295,12 @@ class Cell(object):
 
     @property
     def numeric_value(self):
-        try:
-            return float(self.value)
-        except ValueError:
+        numeric_value = numericise(self.value, default_blank=None)
+
+        # if could not convert, return None
+        if type(numeric_value) == int or type(numeric_value) == float:
+            return numeric_value
+        else:
             return None
 
     @property
