@@ -26,16 +26,7 @@ from google.oauth2.service_account import (
 )
 
 from .exceptions import IncorrectCellLabel, NoValidUrlKeyFound
-
-if sys.version_info.major == 2:
-    import urllib
-elif sys.version_info.major == 3:
-    import urllib.parse as urllib
-
-try:
-    unicode
-except NameError:
-    basestring = unicode = str
+from urllib.parse import quote as uquote
 
 
 MAGIC_NUMBER = 64
@@ -103,11 +94,11 @@ def numericise(
     default_blank="",
     allow_underscores_in_numeric_literals=False,
 ):
-    """Returns a value that depends on the input string:
-        - Float if input can be converted to Float
-        - Integer if input can be converted to integer
-        - Zero if the input string is empty and empty2zero flag is set
-        - The same input string, empty or not, otherwise.
+    """Returns a value that depends on the input:
+        - Float if input is a string that can be converted to Float
+        - Integer if input is a string that can be converted to integer
+        - Zero if the input is a string that is empty and empty2zero flag is set
+        - The unmodified input value, otherwise.
 
     Examples::
 
@@ -121,6 +112,8 @@ def numericise(
     '32'
     >>> numericise("3.1")
     3.1
+    >>> numericise("2,000.1")
+    2000.1
     >>> numericise("", empty2zero=True)
     0
     >>> numericise("", empty2zero=False)
@@ -134,11 +127,14 @@ def numericise(
     >>> numericise(None)
     >>>
     """
-    if value is not None:
+    if isinstance(value, str):
         if "_" in value:
             if not allow_underscores_in_numeric_literals:
                 return value
             value = value.replace("_", "")
+
+        # replace coma separating thousands to match python format
+        value = value.replace(",", "")
         try:
             value = int(value)
         except ValueError:
@@ -426,15 +422,18 @@ def rightpad(row, max_len):
 
 def fill_gaps(L, rows=None, cols=None):
 
-    max_cols = max(len(row) for row in L) if cols is None else cols
-    max_rows = len(L) if rows is None else rows
+    try:
+        max_cols = max(len(row) for row in L) if cols is None else cols
+        max_rows = len(L) if rows is None else rows
 
-    pad_rows = max_rows - len(L)
+        pad_rows = max_rows - len(L)
 
-    if pad_rows:
-        L = L + ([[]] * pad_rows)
+        if pad_rows:
+            L = L + ([[]] * pad_rows)
 
-    return [rightpad(row, max_cols) for row in L]
+        return [rightpad(row, max_cols) for row in L]
+    except ValueError:
+        return []
 
 
 def cell_list_to_rect(cell_list):
@@ -465,7 +464,7 @@ def cell_list_to_rect(cell_list):
 
 
 def quote(value, safe='', encoding='utf-8'):
-    return urllib.quote(value.encode(encoding), safe)
+    return uquote(value.encode(encoding), safe)
 
 
 def absolute_range_name(sheet_name, range_name=None):
@@ -520,7 +519,7 @@ def is_scalar(x):
     >>> is_scalar(set())
     True
     """
-    return isinstance(x, basestring) or not isinstance(x, Sequence)
+    return isinstance(x, str) or not isinstance(x, Sequence)
 
 
 def filter_dict_values(D):
